library textless;

import 'dart:ui';
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:get/get.dart';

extension TextLessStyle on Widget {
  Widget style(TextStyle style,
      {TextOverflow? overflow,
      int? maxLine,
      TextAlign? textAlign,
      double? textScaleFactor}) {
    var text = this as Text;
    return Text(
      text.data!,
      style: text.style!.merge(style),
      overflow: text.overflow ?? overflow,
      maxLines: text.maxLines ?? maxLine,
      softWrap: false,
      textAlign: text.textAlign ?? textAlign,
      textScaleFactor: text.textScaleFactor ?? textScaleFactor,
    );
  }

  Widget get lineThrough =>
      this.style(TextStyle(decoration: TextDecoration.lineThrough));
  Widget get underline =>
      this.style(TextStyle(decoration: TextDecoration.underline));
  Widget get overline =>
      this.style(TextStyle(decoration: TextDecoration.overline));

  Widget color(Color v) => this.style(TextStyle(color: v));
  Widget backgroundColor(Color v) => this.style(TextStyle(backgroundColor: v));
  Widget size(double v) => this.style(TextStyle(fontSize: v));
  Widget height(double v) => this.style(TextStyle(height: v));

  Widget get italic => this.style(TextStyle(fontStyle: FontStyle.italic));
  Widget get thin => this.style(TextStyle(fontWeight: FontWeight.w100));
  Widget get extraLight => this.style(TextStyle(fontWeight: FontWeight.w200));
  Widget get light => this.style(TextStyle(fontWeight: FontWeight.w300));
  Widget get regular => this.style(TextStyle(fontWeight: FontWeight.normal));
  Widget get medium => this.style(TextStyle(fontWeight: FontWeight.w500));
  Widget get semiBold => this.style(TextStyle(fontWeight: FontWeight.w600));
  Widget get bold => this.style(TextStyle(fontWeight: FontWeight.w700));
  Widget get extraBold => this.style(TextStyle(fontWeight: FontWeight.w800));
  Widget get black => this.style(TextStyle(fontWeight: FontWeight.w900));

  Widget get solidLine =>
      this.style(TextStyle(decorationStyle: TextDecorationStyle.solid));
  Widget get dottedLine =>
      this.style(TextStyle(decorationStyle: TextDecorationStyle.dotted));
  Widget get doubledLine =>
      this.style(TextStyle(decorationStyle: TextDecorationStyle.double));
  Widget get wavyLine =>
      this.style(TextStyle(decorationStyle: TextDecorationStyle.wavy));
  Widget get dashedLine =>
      this.style(TextStyle(decorationStyle: TextDecorationStyle.dashed));

  Widget lineColor(Color v) => this.style(TextStyle(decorationColor: v));
  Widget lineThickness(double v) =>
      this.style(TextStyle(decorationThickness: v));

  Widget get alphabeticBaseline =>
      this.style(TextStyle(textBaseline: TextBaseline.alphabetic));
  Widget get ideographicBaseline =>
      this.style(TextStyle(textBaseline: TextBaseline.ideographic));

  Widget fontFamily(String v) => this.style(TextStyle(fontFamily: v));
  Widget letterSpacing(double v) => this.style(TextStyle(letterSpacing: v));
  Widget wordSpacing(double v) => this.style(TextStyle(wordSpacing: v));
  Widget locale(Locale v) => this.style(TextStyle(locale: v));
  Widget foreground(Paint v) => this.style(TextStyle(foreground: v));
  Widget shadows(List<Shadow> v) => this.style(TextStyle(shadows: v));
  Widget fontFeatures(List<FontFeature> v) =>
      this.style(TextStyle(fontFeatures: v));

  Widget get overflowVisible =>
      this.style(TextStyle(), overflow: TextOverflow.visible);
  Widget get overflowClip =>
      this.style(TextStyle(), overflow: TextOverflow.clip);
  Widget get overflowEllipsis =>
      this.style(TextStyle(), overflow: TextOverflow.ellipsis);
  Widget get overflowFade =>
      this.style(TextStyle(), overflow: TextOverflow.fade);

  Widget maxLine(int v) => this.style(TextStyle(), maxLine: v);
  Widget scaleFactor(double v) => this.style(TextStyle(), textScaleFactor: v);
  Widget get alignLeft => this.style(TextStyle(), textAlign: TextAlign.left);
  Widget get alignRight => this.style(TextStyle(), textAlign: TextAlign.right);
  Widget get alignCenter =>
      this.style(TextStyle(), textAlign: TextAlign.center);
  Widget get alignJustify =>
      this.style(TextStyle(), textAlign: TextAlign.justify);
  Widget get alignStart => this.style(TextStyle(), textAlign: TextAlign.start);
  Widget get alignEnd => this.style(TextStyle(), textAlign: TextAlign.end);
}

extension TextLess on String {
  Widget style(TextStyle style) {
    return Text(this, style: style);
  }

  Widget get h1 => Text(this, style: Get.context!.textTheme.headline1);
  Widget get h2 => Text(this, style: Get.context!.textTheme.headline2);
  Widget get h3 => Text(this, style: Get.context!.textTheme.headline3);
  Widget get h4 => Text(this, style: Get.context!.textTheme.headline4);
  Widget get h5 => Text(this, style: Get.context!.textTheme.headline5);
  Widget get h6 => Text(this, style: Get.context!.textTheme.headline6);
  Widget get s1 => Text(this, style: Get.context!.textTheme.subtitle1);
  Widget get s2 => Text(this, style: Get.context!.textTheme.subtitle2);
  Widget get b1 => Text(this, style: Get.context!.textTheme.bodyText1);
  Widget get b2 => Text(this, style: Get.context!.textTheme.bodyText2);
  Widget get btn => Text(this, style: Get.context!.textTheme.button);
  Widget get cap => Text(this, style: Get.context!.textTheme.caption);
  Widget get ol => Text(this, style: Get.context!.textTheme.overline);
}
